"""Morpohosyntactic Dictionary Interface definition."""

from abc import ABC, abstractmethod
from typing import List, Optional, Type, Dict

from src.detections import Detection


class MorphosyntacticDictionary(ABC):
    """Interface for dictionaries with morphosyntactic tags.

    Dictionary allows to replace a detection with anonimized one
    while (trying to) keep the same morphosyntactic form

    Ie.: Janusza -> Marka, Januszowi -> Markowi etc.
    """

    @abstractmethod
    def get_supported_detection_classes(self) -> List[Type[Detection]]:
        """Returns a list of supported detection classess.

        Returns:
            List[Type[Detection]]: List of detection classes that are supported

        """
        pass

    @abstractmethod
    def get_random_replacement(self, original_entry: Detection) -> Optional[str]:
        """Returns a random replacement of original entry.

        Args:
            original_entry (Detection): Detection that should be replaced. Class
                should have MorphosyntacticInfoMixin


        Returns:
            Optional[dict]: Dictionary with replacement and morphosyntactic tag
                The dictionary should have following keys:
                word[str] -> replacement word
                lemma[str] -> lemma of the replacement word
                tag[str] -> morphosyntactic tag of the replacement word
                type[str] -> type of the replacement word

        """
        pass

    def get_random_replacements(
        self, original_entries: List[Detection]
    ) -> List[Optional[Dict[str, str]]]:
        """Returns a list of random replacements of original entries.

        !!! IMPORTANT !!!
        It's assumed that all of the original entries have the same lemma

        Args:
            original_entries (List[Detection]): List of detections that should be
                replaced. All detections should have the same lemma and detection
                type

        Returns:
            Optional[Dict[str, str]]: List of dictionaries with replacement and
                morphosyntactic tag. The dictionary should have following keys:
                word[str] -> replacement word
                lemma[str] -> lemma of the replacement word
                tag[str] -> morphosyntactic tag of the replacement word
                type[str] -> type of the replacement word

        """
        pass
